/*
 * InputHandler.h
 *
 *  Created on: Dec 8, 2013
 *      Author: brian
 */

#ifndef INPUTHANDLER_H_
#define INPUTHANDLER_H_


#include "SDL.h"
#include "Vector2D.h"
#include <vector>

enum mouse_buttons
{
  LEFT = 0,
  MIDDLE = 1,
  RIGHT = 2
};



class InputHandler
{
public:
  static InputHandler* Instance()
  {
    if(s_pInstance == 0)
      {
	s_pInstance = new InputHandler();
      }
    return s_pInstance;
  }
  void update();
  void clean();
  void reset();
  bool quit();

  void initialiseJoysticks();
  bool joysticksInitialised() {
    return m_bJoysticksInitialised;
  }
  int xvalue(int joy, int stick);
  int yvalue(int joy, int stick);

  bool getButtonState(int joy, int buttonNumber)
  {
    return m_buttonStates[joy][buttonNumber];
  }

  bool getMouseButtonState(int buttonNumber)
  {
    return m_mouseButtonStates[buttonNumber];
  }

  Vector2D* getMousePosition()
  {
    return m_mousePosition;
  }

  bool isKeyDown(SDL_Scancode );

  // handle keyboard events
  void onKeyDown();
  void onKeyUp();

  // handle mouse events
  void onMouseMove(SDL_Event& event);
  void onMouseButtonDown(SDL_Event& event);
  void onMouseButtonUp(SDL_Event& event);

  // handle joysticks events
  void onJoystickAxisMove(SDL_Event& event);
  void onJoystickButtonDown(SDL_Event& event);
  void onJoystickButtonUp(SDL_Event& event);

private:
 InputHandler(): m_mousePosition(new Vector2D(0,0)) , m_keystates(0),m_quit(false) {
    for(int i = 0; i < 3; i++)
      {
	m_mouseButtonStates.push_back(false);
      }
  }

  ~InputHandler() {
    // delete anything we created dynamically
    delete m_keystates;
    delete m_mousePosition;


}

  static const int m_joystickDeadZone = 10000;

  static InputHandler* s_pInstance;

  std::vector<SDL_Joystick*> m_joysticks;
  bool m_bJoysticksInitialised;
  bool m_quit;
  std::vector< std::pair<Vector2D*, Vector2D*> > m_joystickValues;
  std::vector<std::vector<bool> > m_buttonStates;
  std::vector<bool> m_mouseButtonStates;
  Vector2D* m_mousePosition;

  //const int* m_keystates;
    // keyboard specific
    const Uint8* m_keystates;


};
typedef InputHandler TheInputHandler;


#endif /* INPUTHANDLER_H_ */
